@@ -1,6 +1,6 @@
 /* ios-dev-stream.c - Streaming IO devices.  */
 
-/* Copyright (C) 2020, 2021 Egeyar Bagcioglu */
+/* Copyright (C) 2020, 2021, 2022 Egeyar Bagcioglu */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,44 +46,48 @@ struct ios_dev_stream
     };
 };
 
-static char *
+static const char *
 ios_dev_stream_get_dev_if_name () {
   return "STREAM";
 }
 
-static int
-ios_dev_stream_handler_normalize (const char *handler, uint64_t flags, char **newhandler)
+static char *
+ios_dev_stream_handler_normalize (const char *handler, uint64_t flags, int *error)
 {
+  char *new_handler = NULL;
   /* TODO handle the case where strdup fails. */
   if (STREQ (handler, IOS_STDIN_HANDLER)
       || STREQ (handler, IOS_STDOUT_HANDLER)
       || STREQ (handler, IOS_STDERR_HANDLER))
     {
-      *newhandler = strdup (handler);
-      if (*newhandler == NULL)
-        return IOD_ENOMEM;
+      new_handler = strdup (handler);
+      if (new_handler == NULL && error)
+        *error = IOD_ENOMEM;
     }
-  return IOD_OK;
+  if (error)
+    *error = IOD_OK;
+  return new_handler;
 }
 
-static int
-ios_dev_stream_open (const char *handler, uint64_t flags, void **dev)
+static void *
+ios_dev_stream_open (const char *handler, uint64_t flags, int *error,
+                     void *data __attribute__ ((unused)))
 {
   struct ios_dev_stream *sio;
-  int error = IOD_ERROR;
+  int internal_error = IOD_ERROR;
 
   sio = malloc (sizeof (struct ios_dev_stream));
   if (!sio)
     {
-      error = IOD_ENOMEM;
-      goto error;
+      internal_error = IOD_ENOMEM;
+      goto err;
     }
 
   sio->handler = strdup (handler);
   if (!sio->handler)
     {
-      error = IOD_ENOMEM;
-      goto error;
+      internal_error = IOD_ENOMEM;
+      goto err;
     }
 
   if (STREQ (handler, IOS_STDIN_HANDLER))
@@ -93,8 +97,8 @@ ios_dev_stream_open (const char *handler, uint64_t flags, void **dev)
       sio->buffer = ios_buffer_init ();
       if (!sio->buffer)
         {
-          error = IOD_ENOMEM;
-          goto error;
+          internal_error = IOD_ENOMEM;
+          goto err;
         }
     }
   else if (STREQ (handler, IOS_STDOUT_HANDLER))
@@ -110,17 +114,20 @@ ios_dev_stream_open (const char *handler, uint64_t flags, void **dev)
       sio->write_offset = 0;
     }
   else
-    goto error;
+    goto err;
 
-  *dev = sio;
-  return IOD_OK;
+  if (error)
+   *error = IOD_OK;
+  return sio;
 
-error:
+err:
   if (sio)
     free (sio->handler);
   free (sio);
 
-  return error;
+  if (error)
+    *error = internal_error;
+  return NULL;
 }
 
 static int
@@ -128,11 +135,15 @@ ios_dev_stream_close (void *iod)
 {
   struct ios_dev_stream *sio = iod;
 
+  /* Do not close std IO files.
+     The user may be in interactive mode.  */
+
   if (sio->flags & IOS_F_READ)
     ios_buffer_free (sio->buffer);
+  free (sio->handler);
   free (sio);
 
-  return 1;
+  return IOD_OK;
 }
 
 static uint64_t
@@ -157,7 +168,7 @@ ios_dev_stream_pread (void *iod, void *buf, size_t count, ios_dev_off offset)
   if (ios_buffer_get_begin_offset (buffer) > offset)
     return IOD_EOF;
 
-  /* If the requsted range is in the buffer, return it. */
+  /* If the requested range is in the buffer, return it. */
   if (ios_buffer_get_end_offset (buffer) >= offset + count)
     return ios_buffer_pread (buffer, buf, count, offset);
 
@@ -178,20 +189,23 @@ ios_dev_stream_pread (void *iod, void *buf, size_t count, ios_dev_off offset)
   do
     {
       read_count = fread (buf + total_read_count,
-                          count - total_read_count,
                           1,
+                          count - total_read_count,
                           sio->file);
       total_read_count += read_count;
     }
   while (total_read_count < count && read_count);
 
   /* Write back to the buffer.  */
-  if (ios_buffer_pwrite (buffer,
-                         buf + read_from_buffer_count,
-                         count - read_from_buffer_count,
-                         ios_buffer_get_end_offset (buffer))
-      || total_read_count < count)
-    return IOD_ERROR;
+  potential_error = ios_buffer_pwrite (buffer,
+                                       buf + read_from_buffer_count,
+                                       count - read_from_buffer_count,
+                                       ios_buffer_get_end_offset (buffer));
+  if (potential_error != IOD_OK)
+    return potential_error;
+
+  if (total_read_count < count)
+    return IOD_EOF;
 
   return IOD_OK;
 }
