@@ -1,6 +1,6 @@
 /* libpoke.c - Implementation of the public services in libpoke.  */
 
-/* Copyright (C) 2020, 2021, 2022 Jose E. Marchesi */
+/* Copyright (C) 2020, 2021, 2022, 2023 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,8 +36,12 @@ struct _pk_compiler
   pkl_compiler compiler;
   pvm vm;
 
-  pkl_ast_node complete_type;
   int status;  /* Status of last API function call. Initialized with PK_OK */
+  /* Data for completion machinery.  */
+  pkl_ast_node complete_type;
+  ios completion_ios;
+  int completion_idx;
+  struct pkl_ast_node_iter completion_iter;
 };
 
 struct pk_term_if libpoke_term_if;
@@ -57,7 +61,7 @@ pk_compiler_new_with_flags (struct pk_term_if *term_if, uint32_t flags)
       || !term_if->hyperlink_fn || !term_if->end_hyperlink_fn)
     return NULL;
 
-  pkc = malloc (sizeof (struct _pk_compiler));
+  pkc = calloc (1, sizeof (struct _pk_compiler));
   if (pkc)
     {
       uint32_t pkl_flags = 0;
@@ -129,23 +133,74 @@ pk_compile_file (pk_compiler pkc, const char *filename,
              : PK_ERROR);
 }
 
+int
+pk_compile_buffer_with_loc (pk_compiler pkc, const char *buffer,
+                            const char *source,
+                            uint32_t line, uint32_t column,
+                            const char **end, pk_val *exit_exception)
+{
+  PK_RETURN (pkl_execute_buffer (pkc->compiler, buffer,
+                                 source, line, column,
+                                 end, exit_exception) ? PK_OK : PK_ERROR);
+}
+
 int
 pk_compile_buffer (pk_compiler pkc, const char *buffer,
                    const char **end, pk_val *exit_exception)
 {
-  PK_RETURN (pkl_execute_buffer (pkc->compiler, buffer,
-                                 end, exit_exception) ? PK_OK : PK_ERROR);
+  return pk_compile_buffer_with_loc (pkc, buffer,
+                                     NULL /* source */,
+                                     1 /* line */,
+                                     1 /* column */,
+                                     end, exit_exception);
+}
+
+int
+pk_compile_statement_with_loc (pk_compiler pkc, const char *buffer,
+                               const char *source,
+                               uint32_t line, uint32_t column,
+                               const char **end, pk_val *valp,
+                               pk_val *exit_exception)
+{
+  pvm_val val;
+
+  if (!pkl_execute_statement (pkc->compiler, buffer,
+                              source, line, column,
+                              end, &val,
+                              exit_exception))
+    PK_RETURN (PK_ERROR);
+
+  if (valp)
+    *valp = val;
+
+  PK_RETURN (PK_OK);
 }
 
 int
 pk_compile_statement (pk_compiler pkc, const char *buffer,
                       const char **end, pk_val *valp,
                       pk_val *exit_exception)
+{
+  return pk_compile_statement_with_loc (pkc, buffer,
+                                        NULL /* source */,
+                                        1 /* line */,
+                                        1 /* column */,
+                                        end, valp, exit_exception);
+}
+
+int
+pk_compile_expression_with_loc (pk_compiler pkc, const char *buffer,
+                                const char *source,
+                                uint32_t line, uint32_t column,
+                                const char **end, pk_val *valp,
+                                pk_val *exit_exception)
 {
   pvm_val val;
 
-  if (!pkl_execute_statement (pkc->compiler, buffer, end, &val,
-                              exit_exception))
+  if (!pkl_execute_expression (pkc->compiler, buffer,
+                               source, line, column,
+                               end, &val,
+                               exit_exception))
     PK_RETURN (PK_ERROR);
 
   if (valp)
@@ -158,16 +213,12 @@ int
 pk_compile_expression (pk_compiler pkc, const char *buffer,
                        const char **end, pk_val *valp, pk_val *exit_exception)
 {
-  pvm_val val;
-
-  if (!pkl_execute_expression (pkc->compiler, buffer, end, &val,
-                               exit_exception))
-    PK_RETURN (PK_ERROR);
-
-  if (valp)
-    *valp = val;
-
-  PK_RETURN (PK_OK);
+  return pk_compile_expression_with_loc (pkc, buffer,
+                                         NULL /* source */,
+                                         1 /* line */,
+                                         1 /* column */,
+                                         end, valp,
+                                         exit_exception);
 }
 
 int
@@ -198,14 +249,64 @@ pk_set_alien_token_fn (pk_compiler pkc, pk_alien_token_handler_fn cb)
 }
 
 static char *
-complete_struct (pk_compiler pkc,
-                 int *idx, const char *x, size_t len, int state)
+complete_attribute (pk_compiler pkc, const char *x, int state)
 {
+#define IDX pkc->completion_idx
+
+  size_t trunk_len;
+  char *ret = NULL;
+  size_t len = strlen (x);
+  static const char *attr_names[] =
+    {
+#define PKL_DEF_ATTR(CODE,NAME) NAME,
+#include "pkl-attrs.def"
+#undef PKL_DEF_ATTR
+      NULL
+    };
+
+  if (state == 0)
+    IDX = 0;
+
+  trunk_len = len - strlen (strrchr (x, '\'')) + 1;
+
+  int i;
+  for (i = IDX; attr_names[i] != NULL; ++i)
+    {
+      const char *attr_name = attr_names[i];
+
+      if (strncmp (x + trunk_len, attr_name, len - trunk_len) == 0)
+        {
+          char *attr;
+
+          if (asprintf (&attr, "%.*s%s", (int) trunk_len, x, attr_name) == -1)
+            {
+              ret = NULL;
+              goto exit;
+            }
+
+          IDX++;
+          ret = attr;
+          goto exit;
+        }
+    }
+
+ exit:
+  return ret;
+
+#undef IDX
+}
+
+static char *
+complete_struct (pk_compiler pkc, const char *x, int state)
+{
+#define IDX pkc->completion_idx
+
   char *ret = NULL;
   pkl_ast_node type = pkc->complete_type;
   pkl_ast_node t;
   size_t trunk_len;
   int j;
+  size_t len = strlen (x);
 
   if (state == 0)
     {
@@ -213,6 +314,8 @@ complete_struct (pk_compiler pkc,
       int back, over;
       char *base;
 
+      IDX = 0;
+
       compiler_env = pkl_get_env (pkc->compiler);
       base = strndup (x, len - strlen (strchr (x, '.')));
 
@@ -236,10 +339,10 @@ complete_struct (pk_compiler pkc,
 
   t = PKL_AST_TYPE_S_ELEMS (type);
 
-  for (j = 0; j < (*idx); j++)
+  for (j = 0; j < IDX; j++)
     t = PKL_AST_CHAIN (t);
 
-  for (; t; t = PKL_AST_CHAIN (t), (*idx)++)
+  for (; t; t = PKL_AST_CHAIN (t), IDX++)
     {
       pkl_ast_node ename;
       char *elem;
@@ -254,6 +357,7 @@ complete_struct (pk_compiler pkc,
           else
             ename = PKL_AST_DECL_NAME (t);
 
+
           if (ename)
             elem = PKL_AST_IDENTIFIER_POINTER (ename);
           else
@@ -269,7 +373,7 @@ complete_struct (pk_compiler pkc,
                   goto exit;
                 }
 
-              (*idx)++;
+              IDX++;
               ret = name;
               goto exit;
             }
@@ -279,6 +383,35 @@ complete_struct (pk_compiler pkc,
  exit:
   pkc->complete_type = type;
   return ret;
+
+#undef IDX
+}
+
+static char *
+complete_decl (pk_compiler pkc, const char *x, int state)
+{
+#define IDX pkc->completion_idx
+#define ITER pkc->completion_iter
+
+  pkl_env env = pkl_get_env (pkc->compiler);
+  if (state == 0)
+    {
+      pkl_env_iter_begin (env, &ITER);
+      IDX = 0;
+    }
+  else
+    {
+      if (pkl_env_iter_end (env, &ITER))
+        IDX++;
+      else
+        pkl_env_iter_next (env, &ITER);
+    }
+
+  size_t len = strlen (x);
+  return pkl_env_get_next_matching_decl (env, &ITER, x, len);
+
+#undef ITER
+#undef IDX
 }
 
 /* This function is called repeatedly by the readline library, when
@@ -294,30 +427,15 @@ char *
 pk_completion_function (pk_compiler pkc,
                         const char *text, int state)
 {
-  char *function_name;
-  static int idx = 0;
-  static struct pkl_ast_node_iter iter;
-  pkl_env env = pkl_get_env (pkc->compiler);
-  if (state == 0)
-    {
-      pkl_env_iter_begin (env, &iter);
-      idx = 0;
-    }
-  else
-    {
-      if (pkl_env_iter_end (env, &iter))
-        idx++;
-      else
-        pkl_env_iter_next (env, &iter);
-    }
-
-  size_t len = strlen (text);
+  /* Note that it is important to handle ' before . here, in order to
+     support completion of attributes in struct fields.  */
+  if ((text[0] != '\'') && (strchr (text, '\'') != NULL))
+    return complete_attribute (pkc, text, state);
 
   if ((text[0] != '.') && (strchr (text, '.') != NULL))
-    return complete_struct (pkc, &idx, text, len, state);
+    return complete_struct (pkc, text, state);
 
-  function_name = pkl_env_get_next_matching_decl (env, &iter, text, len);
-  return function_name;
+  return complete_decl (pkc, text, state);
 }
 
 /* This function provides command line completion when the tag of an
@@ -331,35 +449,30 @@ pk_completion_function (pk_compiler pkc,
    indicate that there are no more such tags.
  */
 char *
-pk_ios_completion_function (pk_compiler pkc __attribute__ ((unused)),
-                            const char *text, int state)
+pk_ios_completion_function (pk_compiler pkc, const char *text, int state)
 {
-  static ios io;
-  if (state == 0)
-    {
-      io = ios_begin ();
-    }
-  else
-    {
-      io = ios_next (io);
-    }
+#define IO pkc->completion_ios
 
   int len  = strlen (text);
+
+  IO = state == 0 ?  ios_begin () : ios_next (IO);
   while (1)
     {
-      if (ios_end (io))
+      if (ios_end (IO))
         break;
 
       char buf[16];
-      snprintf (buf, 16, "#%d", ios_get_id (io));
+      snprintf (buf, 16, "#%d", ios_get_id (IO));
 
       if (strncmp (buf, text, len) == 0)
         return strdup (buf);
 
-      io = ios_next (io);
+      IO = ios_next (IO);
     }
 
   return NULL;
+
+#undef IO
 }
 
 int
@@ -732,12 +845,12 @@ pk_defvar (pk_compiler pkc, const char *varname, pk_val val)
 }
 
 int
-pk_call (pk_compiler pkc, pk_val cls, pk_val *ret, int narg, ...)
+pk_call (pk_compiler pkc, pk_val cls, pk_val *ret,
+         pk_val *exit_exception, int narg, ...)
 {
   pvm_program program;
   va_list ap;
   enum pvm_exit_code rret;
-  pvm_val exit_exception;
 
   /* Compile a program that calls the function.  */
   va_start (ap, narg);
@@ -748,11 +861,10 @@ pk_call (pk_compiler pkc, pk_val cls, pk_val *ret, int narg, ...)
 
   /* Run the program in the poke VM.  */
   pvm_program_make_executable (program);
-  rret = pvm_run (pkc->vm, program, ret, &exit_exception);
+  rret = pvm_run (pkc->vm, program, ret, exit_exception);
 
   pvm_destroy_program (program);
-  PK_RETURN (rret == PVM_EXIT_OK && exit_exception == PVM_NULL
-             ? PK_OK : PK_ERROR);
+  PK_RETURN (rret == PVM_EXIT_OK ? PK_OK : PK_ERROR);
 }
 
 int
@@ -824,6 +936,20 @@ pk_set_omaps (pk_compiler pkc, int omaps_p)
   pkc->status = PK_OK;
 }
 
+int
+pk_autoremap (pk_compiler pkc)
+{
+  pkc->status = PK_OK;
+  return pvm_autoremap (pkc->vm);
+}
+
+void
+pk_set_autoremap (pk_compiler pkc, int autoremap)
+{
+  pvm_set_autoremap (pkc->vm, autoremap);
+  pkc->status = PK_OK;
+}
+
 enum pk_omode
 pk_omode (pk_compiler pkc)
 {
@@ -951,21 +1077,14 @@ pk_set_pretty_print (pk_compiler pkc, int pretty_print_p)
 void
 pk_print_val (pk_compiler pkc, pk_val val, pk_val *exit_exception)
 {
-  pvm_print_val (pkc->vm, val, exit_exception);
-  pkc->status = PK_OK;
-}
+  pk_val printer = pk_decl_val (pkc, "_pkl_print_any");
 
-void
-pk_print_val_with_params (pk_compiler pkc, pk_val val,
-                          int depth, int mode, int base,
-                          int indent, int acutoff,
-                          uint32_t flags,
-                          pk_val *exit_exception)
-{
-  pvm_print_val_with_params (pkc->vm, val,
-                             depth, mode, base,
-                             indent, acutoff, flags,
-                             exit_exception);
+  if (printer == PK_NULL
+      || pk_call (pkc, printer, NULL, exit_exception,
+                  2, val, PVM_NULL /* depth (use default value) */) == PK_ERROR)
+    pkc->status = PK_ERROR;
+  else
+    pkc->status = PK_OK;
 }
 
 static struct ios_dev_if foreign_iod_if;
