@@ -1,6 +1,6 @@
 /* pkl-env.c - Compile-time lexical environments for Poke.  */
 
-/* Copyright (C) 2019, 2020, 2021, 2022 Jose E. Marchesi */
+/* Copyright (C) 2019, 2020, 2021, 2022, 2023 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -131,25 +131,21 @@ register_decl (int top_level_p,
 
   /* Check if DECL is already registered in the given hash table.
 
-     If we are in the global environment and the declaration is for a
-     variable, funcion, or an unit, then we allow "redefining" by
-     changing the name of the previous declaration to "".
+     If we are in the global environment then we allow "redefining" by
+     changing the name of the previous declaration to "", provided
+     that previous declaration is _not_ declared as immutable.
 
      Otherwise we don't register DECL, as it is already defined.  */
 
   found_decl = get_registered (hash_table, name);
   if (found_decl != NULL)
     {
-      int decl_kind = PKL_AST_DECL_KIND (decl);
-
-      if (top_level_p
-          && (decl_kind == PKL_AST_DECL_KIND_VAR
-              || decl_kind == PKL_AST_DECL_KIND_FUNC
-              || decl_kind == PKL_AST_DECL_KIND_UNIT))
+      if (top_level_p && !PKL_AST_DECL_IMMUTABLE_P (found_decl))
         {
           pkl_ast_node decl_name = PKL_AST_DECL_NAME (found_decl);
 
-          free (PKL_AST_IDENTIFIER_POINTER (decl_name));
+          PKL_AST_DECL_PREV_NAME (found_decl)
+            = PKL_AST_IDENTIFIER_POINTER (decl_name);
           PKL_AST_IDENTIFIER_POINTER (decl_name) = strdup ("");
         }
       else
@@ -437,3 +433,37 @@ pkl_env_get_next_matching_decl (pkl_env env, struct pkl_ast_node_iter *iter,
     }
   return NULL;
 }
+
+static void
+pkl_env_rollback_renames_1 (pkl_hash hash_table)
+{
+  int i;
+  for (i = 0; i < HASH_TABLE_SIZE; ++i)
+    {
+      pkl_ast_node t;
+      pkl_ast_node decl = hash_table[i];
+
+      for (t = decl; t; t = PKL_AST_CHAIN2 (t))
+        {
+          pkl_ast_node decl_name = PKL_AST_DECL_NAME (t);
+
+          if (PKL_AST_DECL_PREV_NAME (t))
+            {
+              assert (decl_name
+                      && STREQ (PKL_AST_IDENTIFIER_POINTER (decl_name), ""));
+
+              free (PKL_AST_IDENTIFIER_POINTER (decl_name));
+              PKL_AST_IDENTIFIER_POINTER (decl_name)
+                = PKL_AST_DECL_PREV_NAME (t);
+              PKL_AST_DECL_PREV_NAME (t) = NULL;
+            }
+        }
+    }
+}
+
+void
+pkl_env_rollback_renames (pkl_env env)
+{
+  pkl_env_rollback_renames_1 (env->hash_table);
+  pkl_env_rollback_renames_1 (env->units_hash_table);
+}
