@@ -1,6 +1,6 @@
 /* pkl-parser.h - Parser for Poke.  */
 
-/* Copyright (C) 2019, 2020, 2021, 2022 Jose E. Marchesi */
+/* Copyright (C) 2019, 2020, 2021, 2022, 2023 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -51,6 +51,8 @@ struct pkl_parser
   int in_method_decl_p;
   char *alien_errmsg;
   pkl_ast_loc prev_loc;
+  uint32_t init_line;
+  uint32_t init_column;
 };
 
 /* Public interface.  */
@@ -64,7 +66,10 @@ int pkl_parse_file (pkl_compiler compiler, pkl_env *env, pkl_ast *ast,
                     FILE *fp, const char *fname);
 
 int pkl_parse_buffer (pkl_compiler compiler, pkl_env *env, pkl_ast *ast,
-                      int what, const char *buffer, const char **end);
+                      int what, const char *buffer,
+                      const char *source,
+                      uint32_t line, uint32_t column,
+                      const char **end);
 
 
 
