@@ -1,6 +1,6 @@
 /* pkl-pass.c - Support for compiler passes.  */
 
-/* Copyright (C) 2019, 2020, 2021 Jose E. Marchesi */
+/* Copyright (C) 2019, 2020, 2021, 2022, 2023 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@
           if (phases[i]->CLASS##_##ORDER##_handlers[(DISCR)])           \
             {                                                           \
               int restart;                                              \
+              pkl_ast_node orig_node = node;                            \
                                                                         \
               node                                                      \
                 = phases[i]->CLASS##_##ORDER##_handlers[(DISCR)] (compiler, \
@@ -59,9 +60,10 @@
                                         phases + i + 1,                 \
                                         flags,                          \
                                         level);                         \
-                  /* goto restart */                                    \
                   goto restart;                                         \
                 }                                                       \
+              else if (node != orig_node)                               \
+                goto newnode;                                           \
             }                                                           \
           i++;                                                          \
         }                                                               \
@@ -78,6 +80,7 @@
           if (phases[i]->what##_handler)                                \
             {                                                           \
               int restart;                                              \
+              pkl_ast_node orig_node = node;                            \
               node                                                      \
                 = phases[i]->what##_handler (compiler,                  \
                                              toplevel,                  \
@@ -105,9 +108,10 @@
                                         phases + i + 1,                 \
                                         flags,                          \
                                         level);                         \
-                  /* goto restart */                                    \
                   goto restart;                                         \
                 }                                                       \
+              else if (node != orig_node)                               \
+                goto newnode;                                           \
             }                                                           \
           i++;                                                          \
         }                                                               \
@@ -169,10 +173,6 @@ pkl_call_node_handlers (pkl_compiler compiler,
               /* Unknown operation code.  */
               assert (0);
             }
-
-          /* The node may have been replaced by the handler above.
-             Refresh the code.  */
-          node_code = PKL_AST_CODE (node);
         }
 
       /* Call the phase handlers defined for specific types, in the given
@@ -182,9 +182,6 @@ pkl_call_node_handlers (pkl_compiler compiler,
           int typecode = PKL_AST_TYPE_CODE (node);
 
           PKL_CALL_PHASES (type, ps, typecode);
-          /* The node may have been replaced by the handler above.
-             Refresh the code.  */
-          node_code = PKL_AST_CODE (node);
         }
 
       /* Call the phase handlers defined for node codes, in the given
@@ -224,10 +221,6 @@ pkl_call_node_handlers (pkl_compiler compiler,
               /* Unknown operation code.  */
               assert (0);
             }
-
-          /* The node may have been replaced by the handler above.
-             Refresh the code.  */
-          node_code = PKL_AST_CODE (node);
         }
 
       /* Call the phase handlers defined for specific types, in the given
@@ -237,14 +230,12 @@ pkl_call_node_handlers (pkl_compiler compiler,
           int typecode = PKL_AST_TYPE_CODE (node);
 
           PKL_CALL_PHASES (type, pr, typecode);
-          /* The node may have been replaced by the handler above.
-             Refresh the code.  */
-          node_code = PKL_AST_CODE (node);
         }
     }
   else
     assert (0);
 
+ newnode:
  restart:
  _exit:
   *_dobreak = dobreak;
@@ -307,7 +298,6 @@ pkl_do_pass_1 (pkl_compiler compiler,
                int flags, int level)
 {
   pkl_ast_node node_orig = node;
-  int node_code = PKL_AST_CODE (node);
   int handlers_used = 0;
   int dobreak = 0;
 
@@ -325,6 +315,8 @@ pkl_do_pass_1 (pkl_compiler compiler,
   node = pkl_call_node_handlers (compiler, toplevel, ast, node, payloads, phases,
                                  &handlers_used, child_pos, parent, &dobreak,
                                  PKL_PASS_PRE_ORDER, flags, level);
+  /* Check that no pre-order handler replaced PKL_PASS_NODE.  */
+  assert (node == node_orig);
   if (dobreak)
     goto _exit;
 
@@ -339,14 +331,15 @@ pkl_do_pass_1 (pkl_compiler compiler,
                            payloads, phases, flags, level);
     }
 
-  switch (node_code)
+  switch (PKL_AST_CODE (node))
     {
     case PKL_AST_EXP:
-      PKL_PASS (PKL_AST_EXP_OPERAND (node, 0));
-      if (PKL_AST_EXP_NUMOPS (node) == 2)
-        PKL_PASS (PKL_AST_EXP_OPERAND (node, 1));
-
-      break;
+      {
+        int i;
+        for (i = 0; i < PKL_AST_EXP_NUMOPS (node); ++i)
+          PKL_PASS (PKL_AST_EXP_OPERAND (node, i));
+        break;
+      }
     case PKL_AST_PROGRAM:
       if (PKL_AST_PROGRAM_ELEMS (node))
         PKL_PASS_CHAIN (PKL_AST_PROGRAM_ELEMS (node));
@@ -471,12 +464,16 @@ pkl_do_pass_1 (pkl_compiler compiler,
       if (PKL_AST_STRUCT_TYPE_FIELD_NAME (node))
         PKL_PASS (PKL_AST_STRUCT_TYPE_FIELD_NAME (node));
       PKL_PASS (PKL_AST_STRUCT_TYPE_FIELD_TYPE (node));
+      if (PKL_AST_STRUCT_TYPE_FIELD_SIZE (node))
+        PKL_PASS (PKL_AST_STRUCT_TYPE_FIELD_SIZE (node));
       if (PKL_AST_STRUCT_TYPE_FIELD_CONSTRAINT (node))
         PKL_PASS (PKL_AST_STRUCT_TYPE_FIELD_CONSTRAINT (node));
       if (PKL_AST_STRUCT_TYPE_FIELD_INITIALIZER (node))
         PKL_PASS (PKL_AST_STRUCT_TYPE_FIELD_INITIALIZER (node));
-      if (PKL_AST_STRUCT_TYPE_FIELD_OPTCOND (node))
-        PKL_PASS (PKL_AST_STRUCT_TYPE_FIELD_OPTCOND (node));
+      if (PKL_AST_STRUCT_TYPE_FIELD_OPTCOND_PRE (node))
+        PKL_PASS (PKL_AST_STRUCT_TYPE_FIELD_OPTCOND_PRE (node));
+      if (PKL_AST_STRUCT_TYPE_FIELD_OPTCOND_POST (node))
+        PKL_PASS (PKL_AST_STRUCT_TYPE_FIELD_OPTCOND_POST (node));
       if (PKL_AST_STRUCT_TYPE_FIELD_LABEL (node))
         PKL_PASS (PKL_AST_STRUCT_TYPE_FIELD_LABEL (node));
       break;
@@ -516,7 +513,8 @@ pkl_do_pass_1 (pkl_compiler compiler,
         PKL_PASS_CHAIN (PKL_AST_COMP_STMT_STMTS (node));
       break;
     case PKL_AST_ASS_STMT:
-      PKL_PASS (PKL_AST_ASS_STMT_EXP (node));
+      if (PKL_AST_ASS_STMT_EXP (node))
+        PKL_PASS (PKL_AST_ASS_STMT_EXP (node));
       PKL_PASS (PKL_AST_ASS_STMT_LVALUE (node));
       break;
     case PKL_AST_IF_STMT:
@@ -540,8 +538,7 @@ pkl_do_pass_1 (pkl_compiler compiler,
       PKL_PASS (PKL_AST_LOOP_STMT_ITERATOR_CONTAINER (node));
       PKL_PASS (PKL_AST_LOOP_STMT_ITERATOR_DECL (node));
       break;
-    case PKL_AST_BREAK_STMT:
-    case PKL_AST_CONTINUE_STMT:
+    case PKL_AST_BREAK_CONTINUE_STMT:
       break;
     case PKL_AST_RETURN_STMT:
       if (PKL_AST_RETURN_STMT_EXP (node))
@@ -550,37 +547,59 @@ pkl_do_pass_1 (pkl_compiler compiler,
     case PKL_AST_EXP_STMT:
       PKL_PASS (PKL_AST_EXP_STMT_EXP (node));
       break;
-    case PKL_AST_TRY_CATCH_STMT:
-      PKL_PASS (PKL_AST_TRY_CATCH_STMT_CODE (node));
-      PKL_PASS (PKL_AST_TRY_CATCH_STMT_HANDLER (node));
-      if (PKL_AST_TRY_CATCH_STMT_ARG (node))
-        PKL_PASS (PKL_AST_TRY_CATCH_STMT_ARG (node));
-      if (PKL_AST_TRY_CATCH_STMT_EXP (node))
-        PKL_PASS (PKL_AST_TRY_CATCH_STMT_EXP (node));
+    case PKL_AST_TRY_STMT_BODY:
+      PKL_PASS (PKL_AST_TRY_STMT_BODY_CODE (node));
       break;
-    case PKL_AST_TRY_UNTIL_STMT:
-      PKL_PASS (PKL_AST_TRY_UNTIL_STMT_CODE (node));
-      PKL_PASS (PKL_AST_TRY_UNTIL_STMT_EXP (node));
+    case PKL_AST_TRY_STMT_HANDLER:
+      PKL_PASS (PKL_AST_TRY_STMT_HANDLER_CODE (node));
+      break;
+    case PKL_AST_TRY_STMT:
+      PKL_PASS (PKL_AST_TRY_STMT_BODY (node));
+      if (PKL_AST_TRY_STMT_HANDLER (node))
+        PKL_PASS (PKL_AST_TRY_STMT_HANDLER (node));
+      if (PKL_AST_TRY_STMT_ARG (node))
+        PKL_PASS (PKL_AST_TRY_STMT_ARG (node));
+      if (PKL_AST_TRY_STMT_EXP (node))
+        PKL_PASS (PKL_AST_TRY_STMT_EXP (node));
       break;
     case PKL_AST_RAISE_STMT:
       if (PKL_AST_RAISE_STMT_EXP (node))
         PKL_PASS (PKL_AST_RAISE_STMT_EXP (node));
       break;
-    case PKL_AST_PRINT_STMT_ARG:
-      if (PKL_AST_PRINT_STMT_ARG_EXP (node))
-        PKL_PASS (PKL_AST_PRINT_STMT_ARG_EXP (node));
+    case PKL_AST_ASM_STMT:
+      if (PKL_AST_ASM_STMT_TEMPLATE (node))
+        PKL_PASS (PKL_AST_ASM_STMT_TEMPLATE (node));
+      if (PKL_AST_ASM_STMT_INPUTS (node))
+        PKL_PASS_CHAIN (PKL_AST_ASM_STMT_INPUTS (node));
+      if (PKL_AST_ASM_STMT_OUTPUTS (node))
+        PKL_PASS_CHAIN (PKL_AST_ASM_STMT_OUTPUTS (node));
+      break;
+    case PKL_AST_FORMAT_ARG:
+      if (PKL_AST_FORMAT_ARG_EXP (node))
+        PKL_PASS (PKL_AST_FORMAT_ARG_EXP (node));
+      break;
+    case PKL_AST_FORMAT:
+      PKL_PASS (PKL_AST_FORMAT_FMT (node));
+      if (PKL_AST_FORMAT_TYPES (node))
+        PKL_PASS_CHAIN (PKL_AST_FORMAT_TYPES (node));
+      if (PKL_AST_FORMAT_ARGS (node))
+        PKL_PASS_CHAIN (PKL_AST_FORMAT_ARGS (node));
       break;
     case PKL_AST_PRINT_STMT:
-      if (PKL_AST_PRINT_STMT_FMT (node))
-        PKL_PASS (PKL_AST_PRINT_STMT_FMT (node));
-      if (PKL_AST_PRINT_STMT_TYPES (node))
-        PKL_PASS_CHAIN (PKL_AST_PRINT_STMT_TYPES (node));
-      if (PKL_AST_PRINT_STMT_ARGS (node))
-        PKL_PASS_CHAIN (PKL_AST_PRINT_STMT_ARGS (node));
+      if (PKL_AST_PRINT_STMT_STR_EXP (node))
+        PKL_PASS (PKL_AST_PRINT_STMT_STR_EXP (node));
+      if (PKL_AST_PRINT_STMT_FORMAT (node))
+        PKL_PASS (PKL_AST_PRINT_STMT_FORMAT (node));
       break;
     case PKL_AST_LAMBDA:
       PKL_PASS (PKL_AST_LAMBDA_FUNCTION (node));
       break;
+    case PKL_AST_ASM_EXP:
+      PKL_PASS (PKL_AST_ASM_EXP_TYPE (node));
+      PKL_PASS (PKL_AST_ASM_EXP_TEMPLATE (node));
+      if (PKL_AST_ASM_EXP_INPUTS (node))
+        PKL_PASS_CHAIN (PKL_AST_ASM_EXP_INPUTS (node));
+      break;
     case PKL_AST_INCRDECR:
       PKL_PASS (PKL_AST_INCRDECR_EXP (node));
       if (PKL_AST_INCRDECR_ASS_STMT (node))
@@ -609,6 +628,7 @@ pkl_do_pass_1 (pkl_compiler compiler,
      registered phases in case they are defined.  */
   if (handlers_used == 0)
     PKL_CALL_PHASES_SINGLE(else);
+ newnode:
  restart:
 
   /* If a new node was created to replace the incoming node, increase
