@@ -1,6 +1,6 @@
 /* pkl.c - Poke compiler.  */
 
-/* Copyright (C) 2019, 2020, 2021, 2022 Jose E. Marchesi */
+/* Copyright (C) 2019, 2020, 2021, 2022, 2023 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -76,7 +76,10 @@ struct pkl_compiler
 static int
 pkl_load_rt (pkl_compiler compiler, char *poke_rt_pk)
 {
-  if (!pkl_execute_file (compiler, poke_rt_pk, NULL))
+  pvm_val exit_exception;
+
+  if (!pkl_execute_file (compiler, poke_rt_pk, &exit_exception)
+      || exit_exception != PVM_NULL)
     {
       free (poke_rt_pk);
 
@@ -132,11 +135,13 @@ pkl_new (pvm vm, const char *rt_path, uint32_t flags)
 #define LOAD_STD(NAME)                                          \
   do                                                            \
     {                                                           \
+      pvm_val exit_exception;                                   \
       char *path = pk_str_concat (rt_path, "/" NAME, NULL);     \
       if (!path)                                                \
         goto out_of_memory;                                     \
                                                                 \
-      if (!pkl_execute_file (compiler, path, NULL))             \
+      if (!pkl_execute_file (compiler, path, &exit_exception)   \
+          || exit_exception != PVM_NULL)                        \
         {                                                       \
           free (path);                                          \
           pkl_free (compiler);                                  \
@@ -239,7 +244,7 @@ rest_of_compilation (pkl_compiler compiler,
   };
 
   /* Note that gen does subpasses, so no transformation phases should
-     be invoked in the bakend pass.  */
+     be invoked in the backend pass.  */
   struct pkl_phase *backend_phases[]
     = { &pkl_phase_gen,
         NULL
@@ -299,21 +304,27 @@ rest_of_compilation (pkl_compiler compiler,
 
 int
 pkl_execute_buffer (pkl_compiler compiler,
-                    const char *buffer, const char **end,
+                    const char *buffer,
+                    const char *source,
+                    uint32_t line,
+                    uint32_t column,
+                    const char **end,
                     pvm_val *exit_exception)
 {
   pkl_ast ast = NULL;
   pvm_program program;
   int ret;
   pkl_env env = NULL;
+  pvm_val val;
 
   compiler->compiling = PKL_COMPILING_PROGRAM;
   env = pkl_env_dup_toplevel (compiler->env);
 
   /* Parse the input routine into an AST.  */
   ret = pkl_parse_buffer (compiler, &env, &ast,
-                          PKL_PARSE_PROGRAM,
-                          buffer, end);
+                          PKL_PARSE_PROGRAM, buffer,
+                          source, line, column,
+                          end);
   if (ret == 1)
     /* Parse error.  */
     goto error;
@@ -328,26 +339,31 @@ pkl_execute_buffer (pkl_compiler compiler,
   //  pvm_disassemble_program (program);
   pvm_program_make_executable (program);
 
-  /* Execute the program in the poke vm.  */
-  {
-    pvm_val val;
-    pvm_run (compiler->vm, program, &val, exit_exception);
-    /* Discard the value.  */
-  }
-
+  /* Execute the program in the poke vm.  Note the return value is
+     discarded.  */
+  pvm_run (compiler->vm, program, &val, exit_exception);
   pvm_destroy_program (program);
-  pkl_env_free (compiler->env);
-  compiler->env = env;
+
+  if (*exit_exception == PVM_NULL)
+    {
+      pkl_env_free (compiler->env);
+      compiler->env = env;
+    }
   return 1;
 
  error:
+  pkl_env_rollback_renames (compiler->env);
   pkl_env_free (env);
   return 0;
 }
 
 int
 pkl_execute_statement (pkl_compiler compiler,
-                       const char *buffer, const char **end,
+                       const char *buffer,
+                       const char *source,
+                       uint32_t line,
+                       uint32_t column,
+                       const char **end,
                        pvm_val *val, pvm_val *exit_exception)
 {
   pkl_ast ast = NULL;
@@ -361,7 +377,11 @@ pkl_execute_statement (pkl_compiler compiler,
   /* Parse the input routine into an AST.  */
   ret = pkl_parse_buffer (compiler, &env, &ast,
                           PKL_PARSE_STATEMENT,
-                          buffer, end);
+                          buffer,
+                          source,
+                          line,
+                          column,
+                          end);
   if (ret == 1)
     /* Parse error.  */
     goto error;
@@ -377,13 +397,17 @@ pkl_execute_statement (pkl_compiler compiler,
 
   /* Execute the routine in the poke vm.  */
   pvm_run (compiler->vm, program, val, exit_exception);
-
   pvm_destroy_program (program);
-  pkl_env_free (compiler->env);
-  compiler->env = env;
+
+  if (*exit_exception == PVM_NULL)
+    {
+      pkl_env_free (compiler->env);
+      compiler->env = env;
+    }
   return 1;
 
  error:
+  pkl_env_rollback_renames (compiler->env);
   pkl_env_free (env);
   return 0;
 }
@@ -403,7 +427,11 @@ pkl_compile_expression (pkl_compiler compiler,
    /* Parse the input program into an AST.  */
    ret = pkl_parse_buffer (compiler, &env, &ast,
                            PKL_PARSE_EXPRESSION,
-                           buffer, end);
+                           buffer,
+                           NULL /* source */,
+                           1 /* line */,
+                           1 /* column */,
+                           end);
    if (ret == 1)
      /* Parse error.  */
      goto error;
@@ -419,16 +447,21 @@ pkl_compile_expression (pkl_compiler compiler,
    compiler->env = env;
    pvm_program_make_executable (program);
 
-  return program;
+   return program;
 
  error:
-  pkl_env_free (env);
-  return NULL;
+   pkl_env_rollback_renames (compiler->env);
+   pkl_env_free (env);
+   return NULL;
 }
 
 int
 pkl_execute_expression (pkl_compiler compiler,
-                        const char *buffer, const char **end,
+                        const char *buffer,
+                        const char *source,
+                        uint32_t line,
+                        uint32_t column,
+                        const char **end,
                         pvm_val *val, pvm_val *exit_exception)
 {
   pkl_ast ast = NULL;
@@ -442,7 +475,11 @@ pkl_execute_expression (pkl_compiler compiler,
   /* Parse the input routine into an AST.  */
   ret = pkl_parse_buffer (compiler, &env, &ast,
                           PKL_PARSE_EXPRESSION,
-                          buffer, end);
+                          buffer,
+                          source,
+                          line,
+                          column,
+                          end);
   if (ret == 1)
     /* Parse error.  */
     goto error;
@@ -458,13 +495,17 @@ pkl_execute_expression (pkl_compiler compiler,
 
   /* Execute the routine in the poke vm.  */
   pvm_run (compiler->vm, program, val, exit_exception);
-
   pvm_destroy_program (program);
-  pkl_env_free (compiler->env);
-  compiler->env = env;
+
+  if (*exit_exception == PVM_NULL)
+    {
+      pkl_env_free (compiler->env);
+      compiler->env = env;
+    }
   return 1;
 
  error:
+  pkl_env_rollback_renames (compiler->env);
   pkl_env_free (env);
   return 0;
 }
@@ -478,6 +519,7 @@ pkl_execute_file (pkl_compiler compiler, const char *fname,
   pvm_program program;
   FILE *fp;
   pkl_env env = NULL;
+  pvm_val val;
 
   compiler->compiling = PKL_COMPILING_PROGRAM;
 
@@ -505,19 +547,19 @@ pkl_execute_file (pkl_compiler compiler, const char *fname,
   fclose (fp);
 
   /* Execute the program in the poke vm.  */
-  {
-    pvm_val val;
-
-    pvm_run (compiler->vm, program, &val, exit_exception);
-  }
-
+  pvm_run (compiler->vm, program, &val, exit_exception);
   pvm_destroy_program (program);
-  pkl_env_free (compiler->env);
-  compiler->env = env;
+
+  if (*exit_exception == PVM_NULL)
+    {
+      pkl_env_free (compiler->env);
+      compiler->env = env;
+    }
   return 1;
 
  error:
   fclose (fp);
+  pkl_env_rollback_renames (compiler->env);
   pkl_env_free (env);
   return 0;
 }
@@ -634,6 +676,9 @@ pkl_add_module (pkl_compiler compiler, const char *path)
 {
   const char *module = last_component (path);
 
+  if (pkl_module_loaded_p (compiler, path))
+    return;
+
   if (compiler->num_modules % PKL_MODULES_STEP == 0)
     {
       size_t size = ((compiler->num_modules + PKL_MODULES_STEP)
@@ -722,6 +767,7 @@ pkl_resolve_module (pkl_compiler compiler,
 int
 pkl_load (pkl_compiler compiler, const char *module)
 {
+  pvm_val exit_exception;
   char *module_filename = pkl_resolve_module (compiler,
                                               module,
                                               0 /* filename_p */);
@@ -731,12 +777,12 @@ pkl_load (pkl_compiler compiler, const char *module)
   if (pkl_module_loaded_p (compiler, module_filename))
     return 1;
 
-  if (!pkl_execute_file (compiler, module_filename, NULL))
-    {
-      pkl_add_module (compiler, module_filename);
-      return 0;
-    }
+  if (!pkl_execute_file (compiler, module_filename,
+                         &exit_exception)
+      || exit_exception != PVM_NULL)
+    return 0;
 
+  pkl_add_module (compiler, module_filename);
   return 1;
 }
 
@@ -775,8 +821,8 @@ pvm_type_to_ast_type (pkl_ast ast, pvm_val type)
         return pkl_ast_make_offset_type (ast, base_type, unit);
         break;
       }
-    case PVM_TYPE_ANY:
-      return pkl_ast_make_any_type (ast);
+    case PVM_TYPE_VOID:
+      return pkl_ast_make_void_type (ast);
       break;
     case PVM_TYPE_STRUCT:
       /* XXX writeme */
@@ -845,3 +891,27 @@ pkl_defvar (pkl_compiler compiler,
   pkl_ast_free (ast);
   return 0;
 }
+
+int
+pkl_tracer_p (pkl_compiler compiler)
+{
+  pkl_env compiler_env = compiler->env;
+  pvm_env runtime_env = pvm_get_env (compiler->vm);
+  int back, over;
+  pkl_ast_node decl = pkl_env_lookup (compiler_env,
+                                      PKL_ENV_NS_MAIN,
+                                      "pk_tracer_p",
+                                      &back, &over);
+  pvm_val val = PVM_NULL;
+
+  if (!compiler->bootstrapped)
+    /* For obvious reasons.  ^^ */
+    return 0;
+
+  assert (decl != NULL
+          && PKL_AST_DECL_KIND (decl) == PKL_AST_DECL_KIND_VAR);
+  val = pvm_env_lookup (runtime_env, back, over);
+  assert (PVM_IS_INT (val));
+
+  return PVM_VAL_INT (val);
+}
