@@ -1,6 +1,6 @@
 /* pvm-program.c - PVM programs.  */
 
-/* Copyright (C) 2020, 2021 Jose E. Marchesi */
+/* Copyright (C) 2020, 2021, 2022, 2023 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 #include <assert.h>
 #include <string.h>
 #include <stdio.h> /* For stdout. */
+#include <xalloc.h> /* For xstrdup.  */
 
 #include "jitter/jitter-print.h"
 
@@ -316,6 +317,53 @@ pvm_disassemble_program_nat (pvm_program program)
                            true, JITTER_OBJDUMP, NULL);
 }
 
+char *
+pvm_program_expand_asm_template (const char *str)
+{
+  /* XXX translate str to handle immediates:
+       "foo"
+       u?int<N>M
+       E_inval, etc.
+     then pass the resulting pointer in the string.
+     but beware of 32-bit: pushlo + push32.  */
+
+  size_t expanded_size = 0, q;
+  const char *p;
+  char *expanded;
+
+  /* First, calculate the size of the expanded string.  */
+  for (p = str; *p != '\0'; ++p)
+    {
+      ++expanded_size;
+    }
+
+  /* Allocate the expanded string.  */
+  expanded = xmalloc (expanded_size + 1);
+
+  /* Now build the expanded string.  */
+  for (p = str, q = 0; *p != '\0'; ++p)
+    {
+      assert (q < expanded_size);
+
+      /* ; -> \n */
+      if (*p == ';')
+        expanded[q++] = '\n';
+      else if (*p == '.')
+        expanded[q++] = '$';
+      else
+        expanded[q++] = *p;
+    }
+  expanded[expanded_size] = '\0';
+
+  return expanded;
+}
+
+void
+pvm_program_parse_from_string (const char *str, pvm_program program)
+{
+  pvm_parse_mutable_routine_from_string (str, program->routine);
+}
+
 void
 pvm_disassemble_program (pvm_program program)
 {
