@@ -1,6 +1,6 @@
 /* pvm.h - Poke Virtual Machine.  */
 
-/* Copyright (C) 2019, 2020, 2021, 2022 Jose E. Marchesi */
+/* Copyright (C) 2019, 2020, 2021, 2022, 2023 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -252,7 +252,7 @@ pvm_val pvm_make_array (pvm_val nelem, pvm_val type);
    NFIELDS is an ulong<64> PVM value specifying the number of fields
    in the struct.  This can be ulong<64>0 for an empty struct.
 
-   NMETHODS is an ulong<64> PVM vlaue specifying the number of methods
+   NMETHODS is an ulong<64> PVM value specifying the number of methods
    in the struct.
 
    TYPE is a type PVM value specifying the type of the struct.
@@ -279,6 +279,7 @@ void pvm_print_string (pvm_val string);
 
 pvm_val pvm_ref_struct (pvm_val sct, pvm_val name);
 pvm_val pvm_ref_struct_cstr (pvm_val sct, const char *name);
+void pvm_ref_set_struct_cstr (pvm_val sct, const char *name, pvm_val val);
 
 /* Given a struct value SCT and the name of a field in NAME, return
    the bit-offset of the referred field in BOFF.
@@ -297,8 +298,6 @@ pvm_val pvm_make_string_type (void);
 
 pvm_val pvm_make_void_type (void);
 
-pvm_val pvm_make_any_type (void);
-
 pvm_val pvm_make_array_type (pvm_val type, pvm_val bound);
 
 pvm_val pvm_make_struct_type (pvm_val nfields, pvm_val name,
@@ -497,7 +496,6 @@ enum pvm_exit_code
   E(NO_IOS)                  \
   E(NO_RETURN)               \
   E(OUT_OF_BOUNDS)           \
-  E(MAP_BOUNDS)              \
   E(EOF)                     \
   E(MAP)                     \
   E(CONV)                    \
@@ -532,10 +530,6 @@ enum pvm_exit_code
 #define PVM_E_OUT_OF_BOUNDS_NAME "out of bounds"
 #define PVM_E_OUT_OF_BOUNDS_ESTATUS 1
 
-#define PVM_E_MAP_BOUNDS    5
-#define PVM_E_MAP_BOUNDS_NAME "out of map bounds"
-#define PVM_E_MAP_BOUNDS_ESTATUS 1
-
 #define PVM_E_EOF           6
 #define PVM_E_EOF_NAME "EOF"
 #define PVM_E_EOF_ESTATUS 1
@@ -588,6 +582,10 @@ enum pvm_exit_code
 #define PVM_E_PERM_NAME     "wrong permissions"
 #define PVM_E_PERM_ESTATUS 1
 
+#define PVM_E_STACK        19
+#define PVM_E_STACK_NAME   "invalid stack"
+#define PVM_E_STACK_ESTATUS 1
+
 typedef struct pvm *pvm;
 
 /* Initialize a new Poke Virtual Machine and return it.  */
@@ -602,8 +600,8 @@ void pvm_shutdown (pvm pvm);
 
 pvm_env pvm_get_env (pvm pvm);
 
-/* Print a profiling summary corresponding to the currrent state of
-   the PVM.  */
+/* Print a profiling summary corresponding to the current state of the
+   PVM.  */
 
 void pvm_print_profile (pvm pvm);
 
@@ -614,11 +612,11 @@ void pvm_reset_profile (pvm pvm);
 /* Run a PVM program in a virtual machine.
 
    If the execution of PROGRAM generates a result value, it is put in
-   RES.
+   RES if the argument is not NULL.
 
-   *EXIT_EXCEPTION is set to an exception value if the execution of
-   the program gets interrupted by an unhandled exception.  Otherwise
-   *EXIT_EXCEPTION is set to PK_NULL.
+   If not NULL, *EXIT_EXCEPTION is set to an exception value if the
+   execution of the program gets interrupted by an unhandled
+   exception.  Otherwise *EXIT_EXCEPTION is set to PK_NULL.
 
    This function returns an exit code, indicating whether the
    execution was successful or not.  */
@@ -713,6 +711,11 @@ void pvm_set_odepth (pvm vm, unsigned int odepth);
 unsigned int pvm_oacutoff (pvm vm);
 void pvm_set_oacutoff (pvm vm, unsigned int cutoff);
 
+/* Get/set the `autoremap' flag in the virtual machine.  */
+
+int pvm_autoremap (pvm vm);
+void pvm_set_autoremap (pvm vm, int autoremap);
+
 /* Get/set the compiler associated to a virtual machine.
 
    This compiler is used when the VM needs to build programs and
@@ -727,9 +730,17 @@ void pvm_set_compiler (pvm vm, pkl_compiler compiler);
 
 /* The following function is to be used in pvm.jitter, because the
    system `assert' may expand to a macro and is therefore
-   non-wrappeable.  */
+   non-wrappeable.
 
-void pvm_assert (int expression);
+   EXPRESSION is the predicate that is expected to be non-zero.
+   EXPRESSION_STR is the string representation of the EXPRESSION.
+   FILENAME and LINE are the location information of invocation
+   of this function.  */
+
+void pvm_assert (int expression, const char *expression_str,
+                 const char *filename, int line);
+
+#define PVM_ASSERT(expr) pvm_assert ((expr), #expr, __FILE__, __LINE__)
 
 /* This is defined in the late-c block in pvm.jitter.  */
 
