@@ -1,8 +1,8 @@
-/* A Bison parser, made by GNU Bison 3.6.2.  */
+/* A Bison parser, made by GNU Bison 3.7.5.  */
 
 /* Bison implementation for Yacc-like parsers in C
 
-   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2020 Free Software Foundation,
+   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2021 Free Software Foundation,
    Inc.
 
    This program is free software: you can redistribute it and/or modify
@@ -45,11 +45,11 @@
    define necessary library symbols; they are noted "INFRINGES ON
    USER NAME SPACE" below.  */
 
-/* Identify Bison output.  */
-#define YYBISON 1
+/* Identify Bison output, and Bison version.  */
+#define YYBISON 30705
 
-/* Bison version.  */
-#define YYBISON_VERSION "3.6.2"
+/* Bison version string.  */
+#define YYBISON_VERSION "3.7.5"
 
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
@@ -98,6 +98,7 @@
 static pk_map_parsed_entry
 make_map_entry (const char *name,
                 const char *condition,
+                struct pk_map_parser_loc condition_loc,
                 const char *type,
                 const char *offset)
 {
@@ -116,6 +117,7 @@ make_map_entry (const char *name,
     PK_MAP_PARSED_ENTRY_CONDITION (entry) = xstrdup (condition);
   else
     PK_MAP_PARSED_ENTRY_CONDITION (entry) = NULL;
+  PK_MAP_PARSED_ENTRY_CONDITION_LOC (entry) = condition_loc;
 
   return entry;
 }
@@ -234,7 +236,7 @@ get_tagged_value (tagged_value values, int tag)
 
 /* Error printer.  */
 
-void
+static void
 pk_map_printf_error (struct pk_map_parser *map_parser,
                      YYLTYPE loc, const char *format, ...)
 {
@@ -283,7 +285,7 @@ check_entry_duplicated_tags (struct pk_map_parser *map_parser,
             {
               pk_map_printf_error (map_parser,
                                    value2->loc,
-                                   "duplicated tag");
+                                   "duplicated tag\n");
               return 0;
             }
         }
@@ -301,7 +303,7 @@ pk_map_tab_error (YYLTYPE *llocp, struct pk_map_parser *map_parser, char const *
 }
 
 
-#line 305 "pk-map-tab.c"
+#line 307 "pk-map-tab.c"
 
 # ifndef YY_CAST
 #  ifdef __cplusplus
@@ -324,83 +326,7 @@ pk_map_tab_error (YYLTYPE *llocp, struct pk_map_parser *map_parser, char const *
 #  endif
 # endif
 
-/* Use api.header.include to #include this header
-   instead of duplicating it here.  */
-#ifndef YY_PK_MAP_TAB_PK_MAP_TAB_TAB_H_INCLUDED
-# define YY_PK_MAP_TAB_PK_MAP_TAB_TAB_H_INCLUDED
-/* Debug traces.  */
-#ifndef PK_MAP_TAB_DEBUG
-# if defined YYDEBUG
-#if YYDEBUG
-#   define PK_MAP_TAB_DEBUG 1
-#  else
-#   define PK_MAP_TAB_DEBUG 0
-#  endif
-# else /* ! defined YYDEBUG */
-#  define PK_MAP_TAB_DEBUG 1
-# endif /* ! defined YYDEBUG */
-#endif  /* ! defined PK_MAP_TAB_DEBUG */
-#if PK_MAP_TAB_DEBUG
-extern int pk_map_tab_debug;
-#endif
-
-/* Token kinds.  */
-#ifndef PK_MAP_TAB_TOKENTYPE
-# define PK_MAP_TAB_TOKENTYPE
-  enum pk_map_tab_tokentype
-  {
-    PK_MAP_TAB_EMPTY = -2,
-    PK_MAP_TAB_EOF = 0,            /* "end of file"  */
-    PK_MAP_TAB_error = 256,        /* error  */
-    PK_MAP_TAB_UNDEF = 257,        /* "invalid token"  */
-    ERROR = 258,                   /* ERROR  */
-    SEP = 259,                     /* SEP  */
-    ENTRY = 260,                   /* ENTRY  */
-    TAG = 261,                     /* TAG  */
-    DATA = 262                     /* DATA  */
-  };
-  typedef enum pk_map_tab_tokentype pk_map_tab_token_kind_t;
-#endif
-
-/* Value type.  */
-#if ! defined PK_MAP_TAB_STYPE && ! defined PK_MAP_TAB_STYPE_IS_DECLARED
-union PK_MAP_TAB_STYPE
-{
-#line 263 "pk-map-tab.y"
-
-  char *string;
-  int integer;
-  struct pk_map_parsed_map *map;
-  struct pk_map_parsed_entry *entry;
-  struct tagged_value *tagged_value;
-
-#line 378 "pk-map-tab.c"
-
-};
-typedef union PK_MAP_TAB_STYPE PK_MAP_TAB_STYPE;
-# define PK_MAP_TAB_STYPE_IS_TRIVIAL 1
-# define PK_MAP_TAB_STYPE_IS_DECLARED 1
-#endif
-
-/* Location type.  */
-#if ! defined PK_MAP_TAB_LTYPE && ! defined PK_MAP_TAB_LTYPE_IS_DECLARED
-typedef struct PK_MAP_TAB_LTYPE PK_MAP_TAB_LTYPE;
-struct PK_MAP_TAB_LTYPE
-{
-  int first_line;
-  int first_column;
-  int last_line;
-  int last_column;
-};
-# define PK_MAP_TAB_LTYPE_IS_DECLARED 1
-# define PK_MAP_TAB_LTYPE_IS_TRIVIAL 1
-#endif
-
-
-
-int pk_map_tab_parse (struct pk_map_parser *map_parser);
-
-#endif /* !YY_PK_MAP_TAB_PK_MAP_TAB_TAB_H_INCLUDED  */
+#include "pk-map-tab.h"
 /* Symbol kind.  */
 enum yysymbol_kind_t
 {
@@ -462,6 +388,18 @@ typedef int_least16_t yytype_int16;
 typedef short yytype_int16;
 #endif
 
+/* Work around bug in HP-UX 11.23, which defines these macros
+   incorrectly for preprocessor constants.  This workaround can likely
+   be removed in 2023, as HPE has promised support for HP-UX 11.23
+   (aka HP-UX 11i v2) only through the end of 2022; see Table 2 of
+   <https://h20195.www2.hpe.com/V2/getpdf.aspx/4AA4-7673ENW.pdf>.  */
+#ifdef __hpux
+# undef UINT_LEAST8_MAX
+# undef UINT_LEAST16_MAX
+# define UINT_LEAST8_MAX 255
+# define UINT_LEAST16_MAX 65535
+#endif
+
 #if defined __UINT_LEAST8_MAX__ && __UINT_LEAST8_MAX__ <= __INT_MAX__
 typedef __UINT_LEAST8_TYPE__ yytype_uint8;
 #elif (!defined __UINT_LEAST8_MAX__ && defined YY_STDINT_H \
@@ -559,9 +497,9 @@ typedef int yy_state_fast_t;
 
 /* Suppress unused-variable warnings by "using" E.  */
 #if ! defined lint || defined __GNUC__
-# define YYUSE(E) ((void) (E))
+# define YY_USE(E) ((void) (E))
 #else
-# define YYUSE(E) /* empty */
+# define YY_USE(E) /* empty */
 #endif
 
 #if defined __GNUC__ && ! defined __ICC && 407 <= __GNUC__ * 100 + __GNUC_MINOR__
@@ -742,6 +680,7 @@ union yyalloc
 /* YYNSTATES -- Number of states.  */
 #define YYNSTATES  12
 
+/* YYMAXUTOK -- Last valid token kind.  */
 #define YYMAXUTOK   262
 
 
@@ -786,10 +725,10 @@ static const yytype_int8 yytranslate[] =
 };
 
 #if PK_MAP_TAB_DEBUG
-  /* YYRLINEYYN -- Source line where rule number YYN was defined.  */
+  /* YYRLINE[YYN] -- Source line where rule number YYN was defined.  */
 static const yytype_int16 yyrline[] =
 {
-       0,   290,   290,   300,   302,   306,   345,   347,   352
+       0,   292,   292,   302,   304,   308,   348,   350,   355
 };
 #endif
 
@@ -836,7 +775,7 @@ static const yytype_int16 yytoknum[] =
 #define yytable_value_is_error(Yyn) \
   0
 
-  /* YYPACTSTATE-NUM -- Index in YYTABLE of the portion describing
+  /* YYPACT[STATE-NUM] -- Index in YYTABLE of the portion describing
      STATE-NUM.  */
 static const yytype_int8 yypact[] =
 {
@@ -844,7 +783,7 @@ static const yytype_int8 yypact[] =
       -8,    -8
 };
 
-  /* YYDEFACTSTATE-NUM -- Default reduction number in state STATE-NUM.
+  /* YYDEFACT[STATE-NUM] -- Default reduction number in state STATE-NUM.
      Performed when YYTABLE does not specify something else to do.  Zero
      means the default is an error.  */
 static const yytype_int8 yydefact[] =
@@ -853,19 +792,19 @@ static const yytype_int8 yydefact[] =
        7,     8
 };
 
-  /* YYPGOTONTERM-NUM.  */
+  /* YYPGOTO[NTERM-NUM].  */
 static const yytype_int8 yypgoto[] =
 {
       -8,    -8,    -8,    -8,    -8,    -8
 };
 
-  /* YYDEFGOTONTERM-NUM.  */
+  /* YYDEFGOTO[NTERM-NUM].  */
 static const yytype_int8 yydefgoto[] =
 {
-      -1,     2,     5,     7,     8,    10
+       0,     2,     5,     7,     8,    10
 };
 
-  /* YYTABLEYYPACT[STATE-NUM] -- What to do in state STATE-NUM.  If
+  /* YYTABLE[YYPACT[STATE-NUM]] -- What to do in state STATE-NUM.  If
      positive, shift that token.  If negative, reduce the rule whose
      number is the opposite.  If YYTABLE_NINF, syntax error.  */
 static const yytype_int8 yytable[] =
@@ -878,7 +817,7 @@ static const yytype_int8 yycheck[] =
        7,     4,     0,     5,    -1,     6,    -1,     7
 };
 
-  /* YYSTOSSTATE-NUM -- The (internal number of the) accessing
+  /* YYSTOS[STATE-NUM] -- The (internal number of the) accessing
      symbol of state STATE-NUM.  */
 static const yytype_int8 yystos[] =
 {
@@ -886,13 +825,13 @@ static const yytype_int8 yystos[] =
       13,     7
 };
 
-  /* YYR1YYN -- Symbol number of symbol that rule YYN derives.  */
+  /* YYR1[YYN] -- Symbol number of symbol that rule YYN derives.  */
 static const yytype_int8 yyr1[] =
 {
        0,     8,     9,    10,    10,    11,    12,    12,    13
 };
 
-  /* YYR2YYN -- Number of symbols on the right hand side of rule YYN.  */
+  /* YYR2[YYN] -- Number of symbols on the right hand side of rule YYN.  */
 static const yytype_int8 yyr2[] =
 {
        0,     2,     3,     0,     2,     2,     0,     2,     2
@@ -1039,9 +978,9 @@ yy_symbol_value_print (FILE *yyo,
                        yysymbol_kind_t yykind, YYSTYPE const * const yyvaluep, YYLTYPE const * const yylocationp, struct pk_map_parser *map_parser)
 {
   FILE *yyoutput = yyo;
-  YYUSE (yyoutput);
-  YYUSE (yylocationp);
-  YYUSE (map_parser);
+  YY_USE (yyoutput);
+  YY_USE (yylocationp);
+  YY_USE (map_parser);
   if (!yyvaluep)
     return;
 # ifdef YYPRINT
@@ -1049,7 +988,7 @@ yy_symbol_value_print (FILE *yyo,
     YYPRINT (yyo, yytoknum[yykind], *yyvaluep);
 # endif
   YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
-  YYUSE (yykind);
+  YY_USE (yykind);
   YY_IGNORE_MAYBE_UNINITIALIZED_END
 }
 
@@ -1434,15 +1373,15 @@ static void
 yydestruct (const char *yymsg,
             yysymbol_kind_t yykind, YYSTYPE *yyvaluep, YYLTYPE *yylocationp, struct pk_map_parser *map_parser)
 {
-  YYUSE (yyvaluep);
-  YYUSE (yylocationp);
-  YYUSE (map_parser);
+  YY_USE (yyvaluep);
+  YY_USE (yylocationp);
+  YY_USE (map_parser);
   if (!yymsg)
     yymsg = "Deleting";
   YY_SYMBOL_PRINT (yymsg, yykind, yyvaluep, yylocationp);
 
   YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
-  YYUSE (yykind);
+  YY_USE (yykind);
   YY_IGNORE_MAYBE_UNINITIALIZED_END
 }
 
@@ -1458,7 +1397,7 @@ yydestruct (const char *yymsg,
 int
 yyparse (struct pk_map_parser *map_parser)
 {
-/* The lookahead symbol.  */
+/* Lookahead token kind.  */
 int yychar;
 
 
@@ -1477,42 +1416,37 @@ static YYLTYPE yyloc_default
 YYLTYPE yylloc = yyloc_default;
 
     /* Number of syntax errors so far.  */
-    int yynerrs;
+    int yynerrs = 0;
 
-    yy_state_fast_t yystate;
+    yy_state_fast_t yystate = 0;
     /* Number of tokens to shift before error messages enabled.  */
-    int yyerrstatus;
+    int yyerrstatus = 0;
 
-    /* The stacks and their tools:
-       'yyss': related to states.
-       'yyvs': related to semantic values.
-       'yyls': related to locations.
-
-       Refer to the stacks through separate pointers, to allow yyoverflow
+    /* Refer to the stacks through separate pointers, to allow yyoverflow
        to reallocate them elsewhere.  */
 
     /* Their size.  */
-    YYPTRDIFF_T yystacksize;
+    YYPTRDIFF_T yystacksize = YYINITDEPTH;
 
-    /* The state stack.  */
+    /* The state stack: array, bottom, top.  */
     yy_state_t yyssa[YYINITDEPTH];
-    yy_state_t *yyss;
-    yy_state_t *yyssp;
+    yy_state_t *yyss = yyssa;
+    yy_state_t *yyssp = yyss;
 
-    /* The semantic value stack.  */
+    /* The semantic value stack: array, bottom, top.  */
     YYSTYPE yyvsa[YYINITDEPTH];
-    YYSTYPE *yyvs;
-    YYSTYPE *yyvsp;
+    YYSTYPE *yyvs = yyvsa;
+    YYSTYPE *yyvsp = yyvs;
 
-    /* The location stack.  */
+    /* The location stack: array, bottom, top.  */
     YYLTYPE yylsa[YYINITDEPTH];
-    YYLTYPE *yyls;
-    YYLTYPE *yylsp;
+    YYLTYPE *yyls = yylsa;
+    YYLTYPE *yylsp = yyls;
 
   int yyn;
   /* The return value of yyparse.  */
   int yyresult;
-  /* Lookahead token as an internal (translated) token number.  */
+  /* Lookahead symbol kind.  */
   yysymbol_kind_t yytoken = YYSYMBOL_YYEMPTY;
   /* The variables used to return semantic value and location from the
      action routines.  */
@@ -1533,16 +1467,6 @@ YYLTYPE yylloc = yyloc_default;
      Keep to zero when no symbol should be popped.  */
   int yylen = 0;
 
-  yynerrs = 0;
-  yystate = 0;
-  yyerrstatus = 0;
-
-  yystacksize = YYINITDEPTH;
-  yyssp = yyss = yyssa;
-  yyvsp = yyvs = yyvsa;
-  yylsp = yyls = yylsa;
-
-
   YYDPRINTF ((stderr, "Starting parse\n"));
 
   yychar = PK_MAP_TAB_EMPTY; /* Cause a token to be read.  */
@@ -1554,7 +1478,7 @@ YYLTYPE yylloc = yyloc_default;
     yylloc.first_column = yylloc.last_column = 1;
 }
 
-#line 1558 "pk-map-tab.c"
+#line 1482 "pk-map-tab.c"
 
   yylsp[0] = yylloc;
   goto yysetstate;
@@ -1763,31 +1687,31 @@ yyreduce:
   YY_REDUCE_PRINT (yyn);
   switch (yyn)
     {
-  case 2:
-#line 291 "pk-map-tab.y"
+  case 2: /* map: DATA SEP map_entries  */
+#line 293 "pk-map-tab.y"
                 {
                   /* XXX sort the entries?  Maybe not.  */
                   (yyval.map) = make_map ("XXX", (yyvsp[-2].string), (yyvsp[0].entry));
                   (yyval.map)->loc = (yyloc);
                   map_parser->map = ((yyval.map));
                 }
-#line 1775 "pk-map-tab.c"
+#line 1699 "pk-map-tab.c"
     break;
 
-  case 3:
-#line 301 "pk-map-tab.y"
-                { (yyval.entry) = NULL; }
-#line 1781 "pk-map-tab.c"
-    break;
-
-  case 4:
+  case 3: /* map_entries: %empty  */
 #line 303 "pk-map-tab.y"
-                { (yyval.entry) = map_entry_chainon ((yyvsp[-1].entry), (yyvsp[0].entry)); }
-#line 1787 "pk-map-tab.c"
+                { (yyval.entry) = NULL; }
+#line 1705 "pk-map-tab.c"
     break;
 
-  case 5:
-#line 307 "pk-map-tab.y"
+  case 4: /* map_entries: map_entries map_entry  */
+#line 305 "pk-map-tab.y"
+                { (yyval.entry) = map_entry_chainon ((yyvsp[-1].entry), (yyvsp[0].entry)); }
+#line 1711 "pk-map-tab.c"
+    break;
+
+  case 5: /* map_entry: ENTRY tagged_value_list  */
+#line 309 "pk-map-tab.y"
                   {
                   tagged_value name = get_tagged_value ((yyvsp[0].tagged_value), TAG_NAME);
                   tagged_value type = get_tagged_value ((yyvsp[0].tagged_value), TAG_TYPE);
@@ -1802,11 +1726,11 @@ yyreduce:
                   if (!name || !type || !offset)
                     {
                       if (!name)
-                        pk_map_printf_error (map_parser, (yyloc), "entry lacks a %name");
+                        pk_map_printf_error (map_parser, (yyloc), "entry lacks a %%name\n");
                       else if (!type)
-                        pk_map_printf_error (map_parser, (yyloc), "entry lacks a %type");
+                        pk_map_printf_error (map_parser, (yyloc), "entry lacks a %%type\n");
                       else if (!offset)
-                        pk_map_printf_error (map_parser, (yyloc), "entry lacks an %offset");
+                        pk_map_printf_error (map_parser, (yyloc), "entry lacks an %%offset\n");
                       else
                         assert (0);
 
@@ -1818,28 +1742,29 @@ yyreduce:
 
                   (yyval.entry) = make_map_entry (name->data,
                                        condition ? condition->data : NULL,
+                                       condition ? condition->loc : PK_MAP_NOLOC,
                                        type->data,
                                        offset->data);
                   (yyval.entry)->loc = (yyloc);
                   free_tagged_value_chain ((yyvsp[0].tagged_value));
                 }
-#line 1827 "pk-map-tab.c"
+#line 1752 "pk-map-tab.c"
     break;
 
-  case 6:
-#line 346 "pk-map-tab.y"
+  case 6: /* tagged_value_list: %empty  */
+#line 349 "pk-map-tab.y"
                 { (yyval.tagged_value) = NULL; }
-#line 1833 "pk-map-tab.c"
+#line 1758 "pk-map-tab.c"
     break;
 
-  case 7:
-#line 348 "pk-map-tab.y"
+  case 7: /* tagged_value_list: tagged_value_list tagged_value  */
+#line 351 "pk-map-tab.y"
                 { (yyval.tagged_value) = tagged_value_chainon ((yyvsp[-1].tagged_value), (yyvsp[0].tagged_value)); }
-#line 1839 "pk-map-tab.c"
+#line 1764 "pk-map-tab.c"
     break;
 
-  case 8:
-#line 353 "pk-map-tab.y"
+  case 8: /* tagged_value: TAG DATA  */
+#line 356 "pk-map-tab.y"
           {
             if ((yyvsp[-1].integer) == TAG_UNKNOWN)
               {
@@ -1848,13 +1773,13 @@ yyreduce:
               }
 
             (yyval.tagged_value) = make_tagged_value ((yyvsp[-1].integer), (yyvsp[0].string));
-            (yyval.tagged_value)->loc = (yyloc);
+            (yyval.tagged_value)->loc = (yylsp[0]);
           }
-#line 1854 "pk-map-tab.c"
+#line 1779 "pk-map-tab.c"
     break;
 
 
-#line 1858 "pk-map-tab.c"
+#line 1783 "pk-map-tab.c"
 
       default: break;
     }
@@ -2049,13 +1974,13 @@ yyabortlab:
 yyexhaustedlab:
   yyerror (&yylloc, map_parser, YY_("memory exhausted"));
   yyresult = 2;
-  /* Fall through.  */
+  goto yyreturn;
 #endif
 
 
-/*-----------------------------------------------------.
-| yyreturn -- parsing is finished, return the result.  |
-`-----------------------------------------------------*/
+/*-------------------------------------------------------.
+| yyreturn -- parsing is finished, clean up and return.  |
+`-------------------------------------------------------*/
 yyreturn:
   if (yychar != PK_MAP_TAB_EMPTY)
     {
@@ -2084,7 +2009,7 @@ yyreturn:
   return yyresult;
 }
 
-#line 365 "pk-map-tab.y"
+#line 368 "pk-map-tab.y"
 
 
 /* Parser public functions.  */
