@@ -1,6 +1,6 @@
 /* pk-cmd.c - terminal related stuff.  */
 
-/* Copyright (C) 2019, 2020, 2021 Jose E. Marchesi */
+/* Copyright (C) 2019, 2020, 2021, 2022, 2023 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,21 +18,95 @@
 
 #include <config.h>
 
-#include <stdlib.h> /* For exit.  */
+#include <stdlib.h> /* For exit and getenv.  */
 #include <assert.h> /* For assert. */
 #include <string.h>
-#include <unistd.h> /* For isatty */
+#include <unistd.h> /* For isatty, STDOUT_FILENO */
 #include <textstyle.h>
 #include <assert.h>
 #include <xalloc.h>
+#include <xstrndup.h>
+#include <stdio.h>
+#include <termios.h>
+#include <signal.h>
+#include <sys/ioctl.h>
+#include "terminfo.h"
 
 #include "poke.h"
 #include "pk-utils.h"
 
+/* Several variables related to the pager.  */
+
+/* Terminal control sequence that erases to the end of the current line or of
+   the entire screen.  */
+static const char *erase_line_str;
+
+/* Current screen dimensions.  */
+static int volatile screen_lines = 25;
+static int volatile screen_cols = 80;
+
+static int pager_active_p;
+static int pager_inhibited_p;
+static int nlines = 1;
+
+/* Default style to use when the user doesn't specify a style file.
+   We provide two defaults: one for dark backgrounds (the default) and
+   another for bright backgrounds.  */
+
+static const char *poke_default_style = "poke-dark.css";
+
 /* The following global is the libtextstyle output stream to use to
    emit contents to the terminal.  */
 static styled_ostream_t pk_ostream;
 
+/* Use a signal handler to keep the screen dimensions variables up-to-date.  */
+#if defined SIGWINCH && defined TIOCGWINSZ
+
+static /*_GL_ASYNC_SAFE*/ void
+update_screen_dimensions (void)
+{
+  struct winsize stdout_window_size;
+  if (ioctl (STDOUT_FILENO, TIOCGWINSZ, &stdout_window_size) >= 0
+      && stdout_window_size.ws_row > 0
+      && stdout_window_size.ws_col > 0)
+    {
+      screen_lines = stdout_window_size.ws_row;
+      screen_cols = stdout_window_size.ws_col;
+    }
+}
+
+static void
+sigwinch_handler (int sig)
+{
+  update_screen_dimensions ();
+}
+
+static struct sigaction orig_sigwinch_action;
+
+static void
+install_sigwinch_handler (void)
+{
+  struct sigaction action;
+  action.sa_handler = sigwinch_handler;
+  action.sa_flags = SA_RESTART;
+  sigemptyset (&action.sa_mask);
+  (void) sigaction (SIGWINCH, &action, &orig_sigwinch_action);
+}
+
+static void
+uninstall_sigwinch_handler (void)
+{
+  (void) sigaction (SIGWINCH, &orig_sigwinch_action, NULL);
+}
+
+#else
+
+# define install_sigwinch_handler()
+# define uninstall_sigwinch_handler()
+
+#endif
+
+
 /* Stack of active classes.  */
 
 struct class_entry
@@ -178,6 +252,10 @@ pk_term_init (int argc, char *argv[])
           if (handle_color_option (arg + 8))
             pk_fatal ("handle_color_option failed");
         }
+      else if (strncmp (arg, "--style-dark", 12) == 0)
+        /* This is the default.  See above.  */;
+      else if (strncmp (arg, "--style-bright", 14) == 0)
+        poke_default_style = "poke-bright.css";
       else if (strncmp (arg, "--style=", 8) == 0)
         handle_style_option (arg + 8);
     }
@@ -202,8 +280,9 @@ pk_term_init (int argc, char *argv[])
       || color_mode == color_html)
     {
       /* Find the style file.  */
-      style_file_prepare ("POKE_STYLE", "POKESTYLESDIR", PKGDATADIR,
-                          "poke-default.css");
+      style_file_prepare ("POKE_STYLE", "POKESTYLESDIR",
+                          PKGDATADIR "/poke",
+                          poke_default_style);
     }
   else
     /* No styling.  */
@@ -234,11 +313,48 @@ pk_term_init (int argc, char *argv[])
       register_color (default_bgcolor, -1, -1, -1);
     }
 #endif
+
+  /* Get the terminal dimensions and some terminal control sequences.  */
+  {
+    int done = 0;
+    const char *termtype = getenv ("TERM");
+    if (termtype != NULL && *termtype != '\0')
+      {
+#if defined HAVE_TERMINFO
+        int err = 1;
+        if (setupterm (termtype, STDOUT_FILENO, &err) == 0 || err == 1)
+          {
+            erase_line_str = tigetstr ("el");
+            if (erase_line_str == NULL)
+              erase_line_str = tigetstr ("ed");
+            screen_lines = tigetnum ("lines");
+            screen_cols = tigetnum ("cols");
+            done = 1;
+          }
+#elif defined HAVE_TERMCAP
+        static char termcap_buffer[2048];
+        if (tgetent (termcap_buffer, termtype) > 0)
+          {
+            erase_line_str = tgetstr ("ce");
+            if (erase_line_str == NULL)
+              erase_line_str = tgetstr ("cd");
+            screen_lines = tgetnum ("li");
+            screen_cols = tgetnum ("co");
+            done = 1;
+          }
+#endif
+      }
+    if (!done)
+      update_screen_dimensions ();
+  }
+  install_sigwinch_handler ();
 }
 
 void
 pk_term_shutdown ()
 {
+  uninstall_sigwinch_handler ();
+
   while (active_classes)
     pop_active_class (active_classes->class);
   dispose_color_registry ();
@@ -251,10 +367,113 @@ pk_term_flush ()
   ostream_flush (pk_ostream, FLUSH_THIS_STREAM);
 }
 
+
+void
+pk_term_start_pager (void)
+{
+  pager_active_p = 1;
+  pager_inhibited_p = 0;
+  nlines = 1;
+}
+
+void
+pk_term_stop_pager (void)
+{
+  pager_active_p = 0;
+}
+
+static void
+pk_puts_paged (const char *lines)
+{
+  char *start, *end;
+
+  if (pager_inhibited_p)
+    return;
+
+  start = (char *) lines;
+
+  do
+  {
+    char *line;
+
+    end = strchrnul (start, '\n');
+    line = xstrndup (start, end - start + 1);
+
+    ostream_write_str (pk_ostream, line);
+    free (line);
+    start = end + 1;
+    if (*end != '\0')
+      nlines++;
+
+    if (nlines >= screen_lines)
+      {
+        struct termios old_termios;
+        struct termios new_termios;
+
+        styled_ostream_begin_use_class (pk_ostream, "pager-more");
+        ostream_write_str (pk_ostream, "--More--");
+        styled_ostream_end_use_class (pk_ostream, "pager-more");
+        ostream_flush (pk_ostream, FLUSH_THIS_STREAM);
+
+        /* Set stdin in non-buffered mode.  */
+        tcgetattr (0, &old_termios);
+        memcpy (&new_termios, &old_termios, sizeof (struct termios));
+        new_termios.c_lflag &= ~(ICANON | ECHO);
+        new_termios.c_cc[VTIME] = 0;
+        new_termios.c_cc[VMIN] = 1;
+        tcsetattr (0, TCSANOW, &new_termios);
+
+        /* Wait for a key and process it.  */
+        while (1)
+          {
+            int c = fgetc (stdin);
+
+            if (c == '\n')
+              {
+                nlines--;
+                break;
+              }
+            if (c == ' ')
+              {
+                nlines = 1;
+                break;
+              }
+            if (c == 'q')
+              {
+                pager_inhibited_p = 1;
+                break;
+              }
+
+            /* Ding! 8-) */
+            fprintf (stderr, "\007");
+          }
+
+        /* Restore stdin to buffered-mode.  */
+        tcsetattr (0, TCSANOW, &old_termios);
+
+        if (erase_line_str)
+          {
+            /* Erase --More--  */
+            ostream_write_str (pk_ostream, "\r");
+            ostream_write_str (pk_ostream, erase_line_str);
+            ostream_flush (pk_ostream, FLUSH_THIS_STREAM);
+          }
+        else
+          ostream_write_str (pk_ostream, "\n");
+
+        if (pager_inhibited_p)
+          return;
+     }
+  } while (*end != '\0');
+}
+
 void
 pk_puts (const char *str)
 {
-  ostream_write_str (pk_ostream, str);
+  if (pager_active_p)
+    pk_puts_paged (str);
+  else
+    ostream_write_str (pk_ostream, str);
 }
 
 __attribute__ ((__format__ (__printf__, 1, 2)))
@@ -270,7 +489,7 @@ pk_printf (const char *format, ...)
   assert (r != -1);
   va_end (ap);
 
-  ostream_write_str (pk_ostream, str);
+  pk_puts (str);
   free (str);
 }
 
@@ -283,7 +502,7 @@ pk_vprintf (const char *format, va_list ap)
   r = vasprintf (&str, format, ap);
   assert (r != -1);
 
-  ostream_write_str (pk_ostream, str);
+  pk_puts (str);
   free (str);
 }
 
