@@ -1,6 +1,6 @@
 /* values.c -- Unit tests for the PK values iface in libpoke  */
 
-/* Copyright (C) 2020, 2021 Jose E. Marchesi */
+/* Copyright (C) 2020, 2021, 2022, 2023 Jose E. Marchesi */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -223,7 +223,7 @@ void
 testcase_pk_val_equal_p (const char *filename, const char *sec_code,
                          const char *sec_expr1, const char *sec_expr2)
 {
-  pk_val val1, val2;
+  pk_val val1, val2, exit_exception;
   pk_compiler pkc;
   int equal;
 
@@ -232,13 +232,13 @@ testcase_pk_val_equal_p (const char *filename, const char *sec_code,
   if (!pkc)
     goto error;
 
-  if (pk_compile_buffer (pkc, sec_code, NULL) != PK_OK)
+  if (pk_compile_buffer (pkc, sec_code, NULL, &exit_exception) != PK_OK)
     goto error;
 
-  if (pk_compile_expression (pkc, sec_expr1, NULL, &val1) != PK_OK)
+  if (pk_compile_expression (pkc, sec_expr1, NULL, &val1, &exit_exception) != PK_OK)
     goto error;
 
-  if (pk_compile_expression (pkc, sec_expr2, NULL, &val2) != PK_OK)
+  if (pk_compile_expression (pkc, sec_expr2, NULL, &val2, &exit_exception) != PK_OK)
     goto error;
 
   /*  We should have a way to discriminate if we should check
@@ -260,10 +260,10 @@ testcase_pk_val_equal_p (const char *filename, const char *sec_code,
   else
     {
       printf ("val1:\n");
-      pk_print_val_with_params (pkc, val1, 0, 0, 16, 2, 0, PK_PRINT_F_MAPS);
+      pk_print_val (pkc, val1, NULL /* exit_exception */);
       printf ("\n");
       printf ("val2:\n");
-      pk_print_val_with_params (pkc, val2, 0, 0, 16, 2, 0, PK_PRINT_F_MAPS);
+      pk_print_val (pkc, val2, NULL /* exit_exception */);
       printf ("\n");
       fail (filename);
     }
@@ -276,6 +276,73 @@ error:
   fail (filename);
 }
 
+void
+test_pk_typeof ()
+{
+  pk_val uint32_type = pk_make_integral_type (pk_make_uint (32, 64),
+                                              pk_make_int (1, 32));
+  pk_val res;
+
+  /* If given a type, pk_typeof should return the type itself.  */
+#define TEST "pk_typeof of uint<32>"
+  res = pk_typeof (uint32_type);
+  if (pk_val_equal_p (uint32_type, res))
+    pass (TEST);
+  else
+    fail (TEST);
+#undef TEST
+}
+
+void
+test_pk_struct_ref_set_field_value ()
+{
+  pk_compiler pkc;
+  pk_val exit_exception;
+  pk_val sct, val;
+
+  pkc = pk_compiler_new (&poke_term_if);
+  if (!pkc)
+    {
+      fail ("pk-struct-ref-set-field-value: creating compiler");
+      goto done;
+    }
+
+  if (pk_compile_buffer (pkc,
+                         "type Foo = struct { int foo; int bar; };"
+                         "var f = Foo {};",
+                         NULL, &exit_exception) != PK_OK
+      || exit_exception != PK_NULL)
+    {
+      fail ("pk-struct-ref-set-field-value: compiling buffer");
+      goto done;
+    }
+
+  sct = pk_decl_val (pkc, "f");
+  if (sct == PK_NULL)
+    {
+      fail ("pk-struct-ref-set-field-value: getting value of `f'");
+      goto done;
+    }
+
+  pk_struct_ref_set_field_value (sct, "bar", pk_make_int (666, 32));
+  val = pk_struct_ref_field_value (sct, "bar");
+  if (val == PK_NULL)
+    {
+      fail ("pk-struct-ref-set-field-value: fetching value of field `bar'");
+      goto done;
+    }
+
+  if (pk_val_kind (val) != PK_VAL_INT
+      || pk_int_value (val) != 666)
+    {
+      fail ("pk-struct-ref-set-field-value: invalid value for field `bar'");
+      goto done;
+    }
+
+ done:
+  pk_compiler_free (pkc);
+}
+
 void
 test_pk_val_equal_p ()
 {
@@ -331,6 +398,8 @@ main (int argc, char *argv[])
   test_simple_values ();
   test_simple_values_mapping ();
   test_pk_val_equal_p ();
+  test_pk_typeof ();
+  test_pk_struct_ref_set_field_value ();
 
   totals ();
   return 0;
